#ifndef LIBXMLWRAP_DOCUMENT_H
#define LIBXMLWRAP_DOCUMENT_H

#include <libxml/parser.h>
#include "wrap/xml/string.h"
#include "wrap/xml/parse_error.h"
#include "wrap/xml/node_iterator.h"

namespace xslt
{
class stylesheet;
}

namespace xml
{

class document : boost::noncopyable
{
public:
	~document()
	{
		xmlFreeDoc( ptr_ );
	}

	static std::auto_ptr< document > parse_file( const std::string & file_name )
	{
		xmlDocPtr p = xmlParseFile( file_name.c_str() );
		
		if( !p )
			throw parse_error();

		return std::auto_ptr< document >( new document( p ) );
	}

	static std::auto_ptr< document > parse_string( const std::string & s )
	{
		xmlDocPtr p = xmlParseMemory( s.c_str(), s.size() );
		
		if( !p )
			throw parse_error();

		return std::auto_ptr< document >( new document( p ) );
	}

	typedef node_iterator iterator;
	typedef const_node_iterator const_iterator;

	iterator begin() { return iterator( ptr_->children ); }
	const_iterator begin() const { return const_iterator( ptr_->children ); }
	iterator end() { return iterator(); }
	const_iterator end() const { return const_iterator(); }

	iterator root() { return iterator( xmlDocGetRootElement( ptr_ ) ); }
	const_iterator root() const { return const_iterator( xmlDocGetRootElement( ptr_ ) ); }

private:
	explicit document( xmlDocPtr p ) : ptr_( p ) {}
	xmlDocPtr ptr_;
	friend class xslt::stylesheet;
};

} // namespace xml

#endif // LIBXMLWRAP_DOCUMENT_H
