#ifndef LIBXMLWRAP_NODE_ITERATOR_H
#define LIBXMLWRAP_NODE_ITERATOR_H

#include <iterator>
#include <libxml/parser.h>

#include "wrap/xml/string.h"

namespace xml
{

class node_iterator;
class const_node_iterator;

class node_proxy
{
public:
	node_proxy() : node_( 0 ) {}
	explicit node_proxy( xmlNodePtr n ) : node_( n ) {}

	string name() const { return node_->name; }
	string content() const { return xmlNodeGetContent( node_ ); }
	bool is_text() const { return xmlNodeIsText( node_ ) != 0; }
	bool is_blank() const { return xmlIsBlankNode( node_ ) != 0; }

	node_proxy next() { return node_proxy( node_->next ); }

	node_iterator begin();
	node_iterator end();
	const_node_iterator begin() const;
	const_node_iterator end() const;

private:
	xmlNodePtr node_;
	friend bool operator == ( const node_proxy & n1, const node_proxy & n2 );
};

inline bool operator == ( const node_proxy & n1, const node_proxy & n2 )
{
	return n1.node_ == n2.node_;
}

inline bool operator != ( const node_proxy & n1, const node_proxy & n2 )
{
	return !(n1 == n2);
}

class node_iterator : public std::iterator< std::forward_iterator_tag, xmlNode, ptrdiff_t >
{
public:
	node_iterator() {}
	explicit node_iterator( xmlNodePtr n ) : node_( n ) {}

	void operator ++ ()
	{
		node_ = node_.next();
	}
	
	node_iterator operator ++ ( int )
	{
		node_iterator temp( *this );
		++(*this);
		return temp;
	}
	
	node_proxy & operator * ()
	{
		return node_;
	}
	
	node_proxy * operator -> ()
	{
		return &node_;
	}

private:
	node_proxy node_;
	friend bool operator == ( const node_iterator & i1, const node_iterator & i2 );
};

inline bool operator == ( const node_iterator & i1, const node_iterator & i2 )
{
	return i1.node_ == i2.node_;
}

inline bool operator != ( const node_iterator & i1, const node_iterator & i2 )
{
	return !(i1 == i2);
}

inline node_iterator node_proxy::begin()
{
	return node_iterator( node_->children );
}

inline node_iterator node_proxy::end()
{
	return node_iterator();
}

class const_node_iterator : public std::iterator< std::forward_iterator_tag, xmlNode, ptrdiff_t >
{
public:
	const_node_iterator() {}
	explicit const_node_iterator( xmlNodePtr n ) : node_( n ) {}

	void operator ++ ()
	{
		node_ = node_.next();
	}
	
	const_node_iterator operator ++ ( int )
	{
		const_node_iterator temp( *this );
		++(*this);
		return temp;
	}
	
	const node_proxy & operator * ()
	{
		return node_;
	}
	
	const node_proxy * operator -> ()
	{
		return &node_;
	}

private:
	node_proxy node_;
	friend bool operator == ( const const_node_iterator & i1, const const_node_iterator & i2 );
};

inline bool operator == ( const const_node_iterator & i1, const const_node_iterator & i2 )
{
	return i1.node_ == i2.node_;
}

inline bool operator != ( const const_node_iterator & i1, const const_node_iterator & i2 )
{
	return !(i1 == i2);
}

inline const_node_iterator node_proxy::begin() const
{
	return const_node_iterator( node_->children );
}

inline const_node_iterator node_proxy::end() const
{
	return const_node_iterator();
}

} // namespace xml

#endif // LIBXMLWRAP_NODE_ITERATOR_H
