#ifndef LIBXMLWRAP_STYLESHEET_H
#define LIBXMLWRAP_STYLESHEET_H

#include <wrap/xml/heap_ptr.h>
#include <wrap/xml/document.h>

#include <libxslt/xslt.h>
#include <libxslt/xsltutils.h>
#include <libxslt/transform.h>

#include <wrap/xslt/parse_error.h>
#include <wrap/xslt/apply_error.h>

namespace xslt
{

class stylesheet : boost::noncopyable
{
public:
	~stylesheet()
	{
		xsltFreeStylesheet( ptr_ );
	}

	static std::auto_ptr< stylesheet > parse_file( const std::string & file_name )
	{
		xsltStylesheetPtr p = xsltParseStylesheetFile( reinterpret_cast< const unsigned char * >( file_name.c_str() ) );
		
		if( !p )
			throw parse_error();

		return std::auto_ptr< stylesheet >( new stylesheet( p ) );
	}

	typedef std::map< std::string, std::string > params_type;
	std::auto_ptr< xml::document > apply( xml::document & doc, const params_type & params = params_type() )
	{
		std::vector< const char * > buffer;
		for( params_type::const_iterator i = params.begin(); i != params.end(); ++i )
		{
			buffer.push_back( (*i).first.c_str() );
			buffer.push_back( (*i).second.c_str() );
		}
		buffer.push_back( 0 );
		xmlDocPtr p = xsltApplyStylesheet( ptr_, doc.ptr_, &buffer[0] );
		
		if( !p )
			throw apply_error();

		return std::auto_ptr< xml::document >( new xml::document( p ) );
	}

	std::string save_result_to_string( xml::document & result )
	{
		xmlChar *doc_txt_ptr;
		int doc_txt_len;
		xsltSaveResultToString( &doc_txt_ptr, &doc_txt_len, result.ptr_, ptr_ );

		xml::heap_str str( doc_txt_ptr );
		return std::string( str.get(), str.get() + doc_txt_len );
	}

private:
	explicit stylesheet( xsltStylesheetPtr p ) : ptr_( p ) {}
	xsltStylesheetPtr ptr_;
};

} // namespace xslt

#endif // LIBXMLWRAP_STYLESHEET_H
